package milab.lab06;

import java.util.*;
import java.io.Serializable;

/** Class representing complete, perfect/imperfect information, normal form games
 * 
 * @author Dniel Lszl, Kovcs (dkovacs@mit.bme.hu)
 *
 */
public class Game implements Serializable {

	private static final long serialVersionUID = 5076654959795084395L;
	
	/** Constant identifying the Hawk-Dove game */
	public static final int HAWK_DOVE = 0;
	/** Constant identifying the Prisoners Dilemma game */
	public static final int PRISONERS_DILEMMA = 1;
	/** Constant identifying the Chicken game */
	public static final int CHICKEN = 2;
	/** Constant identifying the Battle of Sexes game */
	public static final int BATTLE_OF_SEXES = 3;
	/** Constant identifying the Cock of the Walk game */
	public static final int LEADER = 4;
	/** Constant identifying the Matching Pennies game */
	public static final int MATCHING_PENNIES = 5;

	/** The type of this game */
	private int game_type;
	/** The name of this game */
	private String game_name;
	/** The names of the strategies in this game */
	private Vector<String[]> strat_names;
	/** The number of players in this game */
	private int player_num;
	/** Is true for symmetric games, else false */
	private boolean is_symmetric;

	public Vector<double[][]> players;
	public double[][] payoff;

	/** Create a predefined game
	 * 
	 * @param type of the game (look at the Game.* constants!)
	 * 
	 */
	public Game(int type) {

		// Set the game-type...
		game_type = type;
		
		// For now we'll just use 2-person 2-2 strategy games...
		player_num			= 2;
		players				= new Vector<double[][]>(player_num);
		strat_names			= new Vector<String[]>(player_num);
		double[][] payoff1	= new double[2][2];
		double[][] payoff2	= new double[2][2];
		
		// Switch between games...
		switch(type) {

			// Strategy 0 is for HAWK, 1 is for DOVE
			case HAWK_DOVE:

				game_name		= "Hawk-Dove";
				is_symmetric	= true;
				strat_names.add(new String[]{"Hawk", "Dove"}); // The strategy names of player 1
				strat_names.add(new String[]{"Hawk", "Dove"}); // The strategy names of player 2
					
				double v = 4.0;				// The value of the resource, that can be gained...
				double c = 2.0;				// The cost of fighting for that resource...

				payoff1[0][0] = (v-c)/2;
				payoff2[0][0] = (v-c)/2;
				payoff1[0][1] = v;
				payoff2[0][1] = 0;			// The payoff of p2 is 0, when p1 plays "0" and p2 plays "1"...
				payoff1[1][0] = 0;			// The payoff of p1 is 0, when p1 plays "1" and p2 plays "0"...
				payoff2[1][0] = v;
				payoff1[1][1] = v/2;
				payoff2[1][1] = v/2;				

				break;
				
			// Strategy 0 is for TALK, 1 is for SILENT
			case PRISONERS_DILEMMA:

				game_name		= "Prisoners' Dilemma";
				is_symmetric	= true;
				strat_names.add(new String[]{"Talk", "Silent"}); // The strategy names of player 1
				strat_names.add(new String[]{"Talk", "Silent"}); // The strategy names of player 2
				
				/*
				payoff1[0][0] = ???;
				payoff2[0][0] = ???;
				payoff1[0][1] = ???;
				payoff2[0][1] = ???;
				payoff1[1][0] = ???;
				payoff2[1][0] = ???;
				payoff1[1][1] = ???;
				payoff2[1][1] = ???;				
				*/
				
				break;
				
			// Strategy 0 is for RECKLESS, 1 is for CHICKEN
			case CHICKEN:

				game_name		= "Chicken";
				is_symmetric	= true;
				strat_names.add(new String[]{"Reckless", "Chicken"}); // The strategy names of player 1
				strat_names.add(new String[]{"Reckless", "Chicken"}); // The strategy names of player 2
				
				/*
				payoff1[0][0] = ???;
				payoff2[0][0] = ???;
				payoff1[0][1] = ???;
				payoff2[0][1] = ???;
				payoff1[1][0] = ???;
				payoff2[1][0] = ???;
				payoff1[1][1] = ???;
				payoff2[1][1] = ???;				
				*/	
				
				break;
				
			// Strategy 0 is for DEFECT, 1 is for COOPERATE
			case BATTLE_OF_SEXES:

				game_name		= "Battle of Sexes";
				is_symmetric	= true;
				strat_names.add(new String[]{"Defect", "Cooperate"}); // The strategy names of player 1
				strat_names.add(new String[]{"Defect", "Cooperate"}); // The strategy names of player 2
				
				// SYMMETRIC VERSION
				/*
				payoff1[0][0] = ???;
				payoff2[0][0] = ???;
				payoff1[0][1] = ???;
				payoff2[0][1] = ???;
				payoff1[1][0] = ???;
				payoff2[1][0] = ???;
				payoff1[1][1] = ???;
				payoff2[1][1] = ???;				
				*/
				
				break;
				
			// Strategy 0 is for GO, 1 is for WAIT
			case LEADER:

				game_name		= "Leader";
				is_symmetric	= true;
				strat_names.add(new String[]{"Go", "Wait"}); // The strategy names of player 1
				strat_names.add(new String[]{"Go", "Wait"}); // The strategy names of player 2
				
				/*
				payoff1[0][0] = ???;
				payoff2[0][0] = ???;
				payoff1[0][1] = ???;
				payoff2[0][1] = ???;
				payoff1[1][0] = ???;
				payoff2[1][0] = ???;
				payoff1[1][1] = ???;
				payoff2[1][1] = ???;				
				*/
				
				break;
				
			// Strategy 0 is for HEADS, 1 is for TAILS
			case MATCHING_PENNIES:
				
				game_name		= "Matching Pennies";
				is_symmetric	= false;
				strat_names.add(new String[]{"Heads", "Tails"}); // The strategy names of player 1
				strat_names.add(new String[]{"Heads", "Tails"}); // The strategy names of player 2
				
				/*
				payoff1[0][0] = ???;
				payoff2[0][0] = ???;
				payoff1[0][1] = ???;
				payoff2[0][1] = ???;
				payoff1[1][0] = ???;
				payoff2[1][0] = ???;
				payoff1[1][1] = ???;
				payoff2[1][1] = ???;				
				*/
				
				break;
				
		} // SWITCH
		
		players.add(payoff1);	// All the possible payoffs of player 1
		players.add(payoff2);	// All the possible payoffs of player 2

	} // End of Game(int) constructor
	
	/** Return the payoff matrix of a player.
	 *  
	 * @param player ...goes from 0 to "unbounded finiteness" (Ivan Bach (tm))
	 * @return payoff matrix of the given player in form of a 2D double[][] array
	 * 
	 */
	public double[][] getPayoffs(int player) {
		
		return players.get(player);
		
	} // End of Game.getPayoffs(int) method

	
	/**
	 * 
	 * Get the name of this game
	 * 
	 * @return the name of this game
	 */
	public String getName() {
		
		return this.game_name;
		
	} // End of Game.getName() method
	
	/**
	 * 
	 * Get the names of the strategies of this game
	 * 
	 * @return the names of the strategies of this game
	 */
	public String[] getStratNames(int player) {
		
		return strat_names.get(player);
		
	} // End of Game.getStratNames() method
	
	public boolean getSymmetry() {
		
		return this.is_symmetric;
		
	} // End of Game.getSymmetry() method

} // End of class Game
