package milab.lab06.GameAgent;

import java.util.*;

import jade.core.Agent;
import jade.core.AID;
import jade.core.behaviours.*;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.content.lang.Codec;
import jade.content.onto.Ontology;
import jade.content.lang.sl.SLCodec;
import jade.domain.DFService;
import jade.domain.FIPANames;
import jade.domain.FIPAException;
import jade.domain.JADEAgentManagement.*;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
//import jade.domain.FIPAAgentManagement.SearchConstraints;
import jade.proto.AchieveREInitiator;
import jade.content.onto.basic.Action;

import milab.lab06.*;

/**
 * GameAgent for evolutionary game theoretic simulations
 * 
 * @author Dniel Lszl, Kovcs (dkovacs@mit.bme.hu)
 *
 */
public class GameAgent extends Agent {

	private static final long serialVersionUID = 1L;

	/** The Game object */
	private Game G;
	/** The maximum number of players/round (should be at least 2 !!!) */
	private int max_players;
	/** The type of the GUI (see GameAgentGui class for GUI*.* named constants!) */
	private int guiType;
	/** A counter for the rounds that passed */
	private int rounds = 0;
	/** A limit for the number rounds */
	private int max_rounds = 250;
	/** The vector of player agents' AID's chosen to play in a round */
	private Vector<AID> playerAgents;
	/** The pairs player agents' AID's formed to play 2-person games */
	private HashMap<AID, AID> playerPairs;
	/** The player ID of the player agent's in a game in round (0 or 1) */
	private HashMap<AID, Integer> playersPID;
	/** The strategic choices received from player agents in a round */
	private HashMap<AID, String> playerAnswers;
	/** The utility of player agents (aggregating over the rounds) */
	private HashMap<AID, Double> playersUtility;
	/** The type of all the player agents ever alive (on our platform) */
	private HashMap<AID, Integer> playersType = new HashMap<AID, Integer>();
	/** The utility of player agents (aggregating over the rounds) */
	private Vector<AID> deadAgents = new Vector<AID>();
	/** A random number generator (with a random seed) */
	private Random r = new Random();
  	/** The GUI showing the progress of the agent population */
  	private GameAgentGui myGui;

  	/**
  	 * 
  	 * Setup the agent: get the start-up arguments, and initialize some variables.
  	 * Then start looking for potential player agents.
  	 * 
  	 */
	protected void setup() {

		// Register to (understand) some languages/codecs and ontologies...
		Codec codec			= new SLCodec();
		Ontology ontology	= JADEManagementOntology.getInstance();
		getContentManager().registerLanguage(codec);
		getContentManager().registerOntology(ontology);

		// If at 1 or 2 start-up arguments were given on the input, then...
		Object[] args = getArguments();
		if (args != null && args.length == 1) {

			G 			= new Game(Integer.parseInt((String)args[0]));
			max_players = 500; // A default value...
			guiType		= GameAgentGui.GUI_TYPES_RATIO;

		} else if (args != null && args.length == 2) {

			G 			= new Game(Integer.parseInt((String)args[0]));
			max_players = Integer.parseInt((String)args[1]);
			guiType		= GameAgentGui.GUI_TYPES_RATIO;
			
		} else if (args != null && args.length >= 3) {

			G 			= new Game(Integer.parseInt((String)args[0]));
			max_players = Integer.parseInt((String)args[1]);
			guiType		= Integer.parseInt((String)args[2]);

		} else { // args == null || args.length == 0...:)

			// Set up some default values...
			G 			= new Game(Game.HAWK_DOVE);
			max_players = 500;
			guiType		= GameAgentGui.GUI_TYPES_RATIO;

		} // IF ARGS.LENGTH > 2

		// Initialize the Map of player-utilities to be an empty HashMap<AID, Double>...
		playersUtility	= new HashMap<AID, Double>();
		
		System.out.println("Hello! Game-agent " + getAID().getName() + " is ready to play the " + G.getName() + " game...");

		// Create and show the GUI...
		myGui = new GameAgentGui(this, guiType);
		myGui.show();

		// Try to wait a few seconds before starting to search for players...
		// This time shall be enough for any initial player-population to start completely
		// after this game agent is born...
		addBehaviour(new WakerBehaviour(this, 10000) {

			private static final long serialVersionUID = -8726861432928954325L;

			protected void onWake() {
				
				// Let's start looking for player agents...
				addBehaviour(new Search4Players());
				
			} // End of WakerBehaviour.onWake()
			
		}); // End of adding a WakerBehaviour to add a Search4Players behaviour...

	} // End of GameAgent.setup()


	/**
	 * 
	 * Print a goodbye message, dispose the GUI, and let the JADE framework do the rest...
	 * 
	 */
	protected void takeDown() {

		// Printout a dismissal message
		System.out.println("Game-agent " + getAID().getName() + " terminating...");

	    // We don't have to deregister from the yellow pages, since we haven't registered... Invisible agent:)
	    /*try {
	    	
	      DFService.deregister(this);
	      
	    } catch (FIPAException fe) {
	    	
	      fe.printStackTrace();
	      
	    }*/
		
	  	// Close the GUI
	  	myGui.dispose();
		
	} // End of GameAgent.takeDown()

	/**
	 * JADE behaviour: search for potential player agents for a round...
	 * 
	 * @author Dniel Lszl, Kovcs (dkovacs@mit.bme.hu)
	 *
	 */
	private class Search4Players extends OneShotBehaviour {

		private static final long serialVersionUID = 1L;

		public void action() {
			
			System.out.println("Trying to locate players for round " + rounds + "...");

			// Create the search template for the DF agent...
			DFAgentDescription template = new DFAgentDescription();
			ServiceDescription sd		= new ServiceDescription();
			sd.setType("player");
			template.addServices(sd);

			// Create some search constraints...
			//SearchConstraints sc		= new SearchConstraints();
			//sc.setMaxResults(new Long(200)); // The limit on the number of search-results...
			
			try {

				// (Re)initialize a vector for players to be chosen in this round...
        		playerAgents = new Vector<AID>();

        		AID a;

        		// Lets generate a random int between 2 and max_players: the number of players/round!
        		// We expect max_players to be at least 2............................................
        		int max_round_players = r.nextInt(max_players-1) + 2;

				// Query the DF agent for players...
				DFAgentDescription[] result = DFService.search(myAgent, template);
				//DFAgentDescription[] result = DFService.search(myAgent, template, sc);

				// Initialize a vector for all the players found in this round...
				Vector<AID> allPlayerAgents = new Vector<AID>(result.length);

				// Convert the result array to a vector...
				for (int i = 0; i < result.length; ++i) {
					
					allPlayerAgents.add(result[i].getName());
					
				} // FOR RESULT
				
				// Let's make a copy of playersUtility for Thread-Safety reasons!!!...
        		HashMap<AID, Double> playersUtility_copy = new HashMap<AID, Double>();
        		Iterator<AID> pu = playersUtility.keySet().iterator();
    			
        		// Do the copying............
        		while(pu.hasNext()) {

    				a = pu.next();
    				playersUtility_copy.put(a, playersUtility.get(a));

    			} // WHILE HAS NEXT

    			// Now let's check if every agent - whose utility is stored (in playersUtility) - is still alive!
          		Iterator<AID> puc = playersUtility_copy.keySet().iterator();

          		while(puc.hasNext()) {

    				a = puc.next();
    				
    				// If the agent disappeared (e.g. died), then...
    				if (!allPlayerAgents.contains(a)) {

    					playersUtility.remove(a);

    				} // IF NOT CONTAINS
	
    			} // WHILE HAS NEXT
     		
        		// Choose players for this round...
        		for (int i = 0; i < result.length; ++i) {

        			// Get the AID of the found agent...
        			a = result[i].getName();
        			
        			// If the agent isn't on the deadlist, then...
        			// -------------------------------------------------------------------------------
        			// It may happen, that a player, after being previously notified of his below-zero
        			// utility, and thus starts to terminate, does it "too slowly", and so we find him/her
        			// still registered to the platform, when querying the DF for potential players for the
        			// next round. This could cause a problem, because we could handle this agent as a
        			// player, and thus send him/her an action-request, although he/she is terminating
        			// (or is terminated), and certainly won't answer - kind of a deadlock...
        			// We could implement timeout's (waiting here and there), but this would decrease
        			// the overall system performance significantly. Besides, this is the reason behind
        			// letting players terminate themselves instead of asking the AMS to terminate them.
        			// 1 more thing: this approach excludes resurrection... ;-) That is: if an agent, who
        			// is known to be dead, is found when querying the DF (i.e. when a new agent with the
        			// same local name was started (maybe by a user) after the previous one terminated),
        			// then we won't handle such an agent as a potential player.
        			// ---------------------------------------------------------
        			if (!deadAgents.contains(a)) {

	        			// If we meet this agent for the first time, then...
	        			if(!playersUtility.containsKey(a)/* && playersUtility.size() < max_players*//*max_round_players*/) { // !!!!!!

	        				// If the agent has services defined...
	        				Iterator<ServiceDescription> asdi = result[i].getAllServices();
	            			if (asdi.hasNext()) {

	            				// The first service-name of the agent of service-type "player" is the "typeID initialUtility" string...
	            				StringTokenizer stok = new StringTokenizer(((ServiceDescription)asdi.next()).getName(), " ");
	            				
	            				// Let's properly initialize the typeID of this agent-player...
	            				playersType.put(a, new Integer(stok.nextToken()));

	            				// Let's properly initialize the utility of this agent-player...
	            				playersUtility.put(a, new Double(stok.nextToken()));
	            				
	            			} else { // Not proper registration...
	            				
	            				// Let's give it a default type! This way, for example, when the GUI considers
	    	        			// the type of this player for choosing a color for him/her on the chart, then
	    	        			// those players, who are in playersUtility, but never played before, will have
	    	        			// a GRAY color by default. Sometime later, when they perhaps become chosen to
	    	        			// play, and inform us of their type, the GUI will be updated accordingly...
	            				playersType.put(a, PlayerType.UNKNOWN);
	            				
	            				// ...an initial utility (which may not be true)...
	            				playersUtility.put(a, new Double(0));
	            				
	            			} // IF SERVICEDESCRIPTION.HASNEXT
	            			
	        			} // IF PLAYERSUTILITY.CONTAINSKEY && PU.SIZE < MAX

	        			// Every possible/allowed player shall play in this round... 
	        			if (playersUtility.containsKey(a) /*r.nextInt(2) == 1 && */) {

	        				// ...choose this agent to play in this round!
	        				playerAgents.add(a);

	        			} // IF SELECTED (this could be modified to a rulette wheel, maybe)
	        			
        			} // IF NOT DEAD

        		} // FOR RESULT

        		System.out.println("- " + playerAgents.size() + "/" + playersUtility.size() + "/" + result.length + " players chosen to play in this round...");
        		// If the number of chosen players is odd, then - because of pairing - one will be left out.

			} catch (FIPAException fe) {

				fe.printStackTrace();

			}
			
			if (playersUtility.size() < max_players && rounds < max_rounds) { // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
				
				// If at least 2 players were chosen to play in this round, then...
				if (playerAgents.size() > 1) {
	
	        		// Initialize the Map of players' answers in this round...
	        		playerAnswers = new HashMap<AID, String>(playerAgents.size());
	
	        		// Pair the agents (give value to playerPairs)...
					makePairs();
					
					// If the Search4Players behaviour is called first time before a play-execution...
					if (rounds == 0) {
						
						  // Update the GUI with the initial utilities of the players...
						  myGui.updateChart(guiType, Integer.toString(rounds), playersUtility, playersType);
						
					}
	
					// Let's execute a round...
					myAgent.addBehaviour(new ExecutePlays());
	
				} else {
	
					// Let's start over, and look for potential players for a round...
					myAgent.addBehaviour(new Search4Players());
	
				} // IF >= 2 PLAYERS WERE CHOSEN TO PLAY IN THIS ROUND

			} else {
				
				System.out.println("Finished...");

			} // IF ROUNDS < MAX_ROUNDS
				
	      } // End of GameAgent.Search4Players.action() method

	} // End of GameAgent.Search4Players behaviour

	/**
	 * JADE behaviour: execute a round, i.e. ask players to act, then wait for their reply (their action),
	 * calculate their outcome, and finally notify them about this.
	 * 
	 * @author Dniel Lszl, Kovcs (dkovacs@mit.bme.hu)
	 *
	 */
	private class ExecutePlays extends Behaviour {

		private static final long serialVersionUID = 1L;
		
		// The counter of replies receiver in this round from the current player agents. Initially zero...
		private int repliesCnt = 0;
		
		// The initial state of this behaviour...
		private int step = 0;			

		public void action() {
			
			switch (step) {

		    	case 0:

		    		// Send an appropriate action-request to all (at least 2) player agents, who have a pair
		    		for (int i = 0; i < playerAgents.size(); ++i) {

		    			// Let's get the i-th agent (say player i) playing in this round...
		    			AID pi = playerAgents.get(i);
		    			
		    			// Let's try to get the pair of player i...
		    			AID pj = playerPairs.get(pi);

		    			// If player i has a pair (player j), then...
		    			if (pj != null) {

		    				// Two things are sent to the agents:
			    			// ----------------------------------
			    			// (1)	The number of the player/role they assume in the game
			    			// (2)  The name of the opponent against whom they'll play
		    				
		    				// Construct an ACTION-REQUEST message to player i...
			    			ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
			    			msg.addReceiver(pi);
		    				msg.setContent(playersPID.get(pi).toString() + " " + pj.getLocalName());
				    		msg.setConversationId("action-info");

				    		// Send the ACTION-REQUEST message to player i...
				    		send(msg);

		    			} // IF Pi HAS ALREADY A PAIR: Pj

		    		} // FOR CHOSEN PLAYERS

		    		// Let's continue to the next step/state: wait for players' reply...
			   		step = 1;

		    		break;

		    	case 1:

		    		// Receive a message (describing the current actions of the player agents)
		    		ACLMessage reply = myAgent.receive(MessageTemplate.MatchConversationId("action-info"));

		    		// If a message was received, then...
		    		if (reply != null) {

		    			AID pi = reply.getSender();
		    			StringTokenizer stok = new StringTokenizer(reply.getContent(), " ");
		    			
		    			// If an appropriate INFORM message was received from one of the players of the current round, then...
		    			if (reply.getPerformative() == ACLMessage.INFORM && stok.countTokens() == 3 && playerAgents.contains(pi)) {

		    				// An appropriate reply was received
		    				repliesCnt++;
		    				
		    				// Player i says that - in this round - he/she wants to do...
		    				String piAction = stok.nextToken();
		    				playerAnswers.put(pi, piAction);
		    				
		    				// Player i says that - at the beginning of this round - his/her utility is...
		    				playersUtility.put(pi, new Double(stok.nextToken()));
		    				
		    				// If the typeID of this agent-player was unknown until yet, then...
		    				if (playersType.get(pi).intValue() == PlayerType.UNKNOWN)
		    												playersType.put(pi, new Integer(stok.nextToken()));

		    				// Let's try to get the pair of player i...
		    				AID pj = playerPairs.get(pi);

		    				// If player i has a pair (player j), then...
		    				if (pj != null) {

		    					String pjAction = playerAnswers.get(pj);

		    					// If player j had told us already, what he/she wants to do in this round, then...
		    					if (pjAction != null) {	// If Pj already chose a strategy

		    						// ----------------------------------------------
					    			// Six things are sent back to each player-agent:
					    			// ----------------------------------------------
					    			// (1)  The name of the opponent against whom they've played
					    			// (2)	The number of the player/role the opponent assumed in the play
					    			// (3)	The number of the strategy the opponent has played in the play
		    						// (4)	The number of the player/role they have assumed in the play
					    			// (5)	The number of the strategy they have played during the play
					    			// (6)	Their respective payoff after the play

		    						// Let's get the roleID of the players...
		    						int piRole	= playersPID.get(pi).intValue();
		    						int pjRole	= playersPID.get(pj).intValue();

		    						// Let's calculate the respective payoff of each player (i.e. execute the play)...
		    						int s0, s1;
		    						if (piRole == 0) {

		    							s0 = Integer.parseInt(piAction);
		    							s1 = Integer.parseInt(pjAction);

		    						} else {

		    							s0 = Integer.parseInt(pjAction);
		    							s1 = Integer.parseInt(piAction);

		    						}
		    						double piPayoff = new Double((G.getPayoffs(piRole))[s0][s1]).doubleValue();
		    						double pjPayoff = new Double((G.getPayoffs(pjRole))[s0][s1]).doubleValue();

		    						// Let's get the actual utility of the players...
		    						double piUtility = playersUtility.get(pi).doubleValue();
		    						double pjUtility = playersUtility.get(pj).doubleValue();

		    						// Let's increment the aggregated utility of the players respective to their current payoff...
		    						piUtility += piPayoff;
		    						pjUtility += pjPayoff;

		    						// If the new utility of players went below zero, then...
		    						if (piUtility < 0) deadAgents.add(pi);
		    						if (pjUtility < 0) deadAgents.add(pj);

	    							// Let's store the new utility values...
	    							playersUtility.put(pi, new Double(piUtility));
	    							playersUtility.put(pj, new Double(pjUtility));

	    							// Can the players reproduce?
	    							String reprok;
	    							
	    							// Don't forget, that when there is a little bit less, than the
	    							// maximum of players, and we allow them to reproduce, there may
	    							// happen to be more agents created on the platform, than the maximum...
	    							if (playersUtility.size() >= max_players) reprok = "0"; else reprok = "1";
	    							
		    						// Let's construct the replies to the players...
		    						ACLMessage msgi = new ACLMessage(ACLMessage.INFORM);
		    						msgi.addReceiver(pi);
		    						msgi.setContent(pj.getLocalName() + " " + pjRole + " " + pjAction + " " + piRole + " " + piAction + " " + piPayoff + " " + reprok);

						    		ACLMessage msgj = new ACLMessage(ACLMessage.INFORM);
						    		msgj.addReceiver(pj);
						    		msgj.setContent(pi.getLocalName() + " " + piRole + " " + piAction + " " + pjRole + " " + pjAction + " " + pjPayoff + " " + reprok);

		    						// Let's notify the players of the outcome of the play...
						    		send(msgi);
						    		send(msgj);				    		

		    					} // IF PjACTION != NULL

		    				} // IF Pj != NULL

		    				// If every member of every pair had sent us his/her reply, then...
		    				if (repliesCnt >= playerPairs.size()) {

		    					// We've received all replies
		    					step = 2;

		    				} // IF REPLIESCNT >= LENGTH

		    			} // IF PERFORMATIVE == INFORM && CONTENT_SYNTAX_OK && PLAYER IS NOT A "HACKER" :)

		    		} else {

	    				block();

	    			} // IF REPLY == NULL
	     
		    		break;

	    } // End of SWITCH       

	  } // End of GameAgent.ExecutePlays.action()

	  public boolean done() {
	
		  if (step == 2) {

			  // Increment the round-number counter...
			  rounds++;
			
			  // Update the GUI with the actual utilities of the players in this round...
			  myGui.updateChart(guiType, Integer.toString(rounds), playersUtility, playersType);
			
			  // Let's start over, and look for potential players for the next round...
			  myAgent.addBehaviour(new Search4Players());
			
			  // Deactivate this behaviour...
			  return true;

		  } else {

			  // Let this behaviour be active...
			  return false;
 
		  }

	  } // End of GameAgent.ExecutePlays.done()

	} // End of GameAgent.ExecutePlays behaviour


	/** Pair the player agents, and associate a playerID/roleID to every one of them. 
	 *  The roleID defines which player/role the agent will have to assume in the game. */
	private void makePairs() {

		// At first all of these players are unpaired...
		int unpaired_player_num = playerAgents.size();
		
		// Make a local copy of playerAgents to manipulate it freely...
		Vector<AID> players	= new Vector<AID>(unpaired_player_num);
		
		// Do the copying............
		for (int i = 0; i < unpaired_player_num; i++) players.add(playerAgents.get(i));

		playerPairs = new HashMap<AID, AID>();
		playersPID	= new HashMap<AID, Integer>();

		while (unpaired_player_num > 1) {

			int i = r.nextInt(unpaired_player_num);
			int j = r.nextInt(unpaired_player_num);

			if (i != j) {

				AID pi = players.get(i);
				AID pj = players.get(j);

				playerPairs.put(pi, pj);
				playerPairs.put(pj, pi);

				// Because there is now always 2 players in a game
				int x = r.nextInt(2); 

				playersPID.put(pi, new Integer(x));
				playersPID.put(pj, new Integer(1-x));

				players.remove(i);
				
				// If by removing pi pj shifted to the left by 1, then...
				if (i < j) {

					players.remove(j-1);

				} else {

					players.remove(j);

				}

				unpaired_player_num -= 2;

			} // I != J

		} // WHILE UPN >= 2

	} // End of GameAgent.makePairs()

	/**
	 * Request the termination of an agent from the AMS
	 * 
	 * @param a The agent to kill.
	 */
	private void kill (AID a) {

		KillAgent ka		= new KillAgent();
		ka.setAgent(a);
		Action actExpr		= new Action(getAMS(), ka);
		ACLMessage request	= new ACLMessage(ACLMessage.REQUEST);
		request.addReceiver(getAMS());
		request.setOntology(JADEManagementOntology.getInstance().getName());
		request.setLanguage(FIPANames.ContentLanguage.FIPA_SL);
		request.setProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST);

		try {

			getContentManager().fillContent(request, actExpr);
			addBehaviour(new AchieveREInitiator(this, request) {

				private static final long serialVersionUID = 1L;

				protected void handleInform(ACLMessage inform) {

					System.out.println("Agent successfully created");

				}

				protected void handleFailure(ACLMessage failure) {

					System.out.println("Error creating agent.");

				}

			}); // End of inner anonym AchieveREInitiator behaviour class

		} catch (Exception e) {

			e.printStackTrace();

		} // END OF TRY-CATCH

	} // End of GameAgent.kill() method

} // End of class GameAgent
