package milab.lab06.PlayerAgent;

import java.util.Random;
import java.util.StringTokenizer;
import java.util.Vector;

import jadex.runtime.*;
import jadex.util.Tuple;
import milab.lab06.*;

/**
 * JADEX player agent's play plan
 * 
 * @author Dniel Lszl, Kovcs (dkovacs@mit.bme.hu)
 *
 */
public class PlayerPlayPlan extends Plan {

	private static final long serialVersionUID = 1L;

	/** A random number generator (with a random seed) */
	private Random r = new Random();

	public PlayerPlayPlan() {

		//getLogger().info("Created: "+this);

	} // End of PlayerPlayPlan.PlayerPlayPlan() constructor

	public void body() {

		// Get the contents of the message(event) - REQUEST from GameAgent - that caused the execution of this plan...
		StringTokenizer stok = new StringTokenizer(	(String)((IMessageEvent)getInitialEvent()).getContent(), " ");
		
		// If the structure of the content is right, then...
		if(stok.countTokens() == 2) {

			Game G			= new Game(((Integer)getBeliefbase().getBelief("G").getFact()).intValue());
			int pid			= Integer.parseInt(stok.nextToken());
			int tid			= ((Integer)getBeliefbase().getBelief("myType").getFact()).intValue();
			String opponent = stok.nextToken();

			// Reply with an INFORM message (about the chosen strategy, actual utility, and type)...
			sendMessage(((IMessageEvent)getInitialEvent()).createReply("inform", (chooseStrategy(G, pid, tid, opponent) + " " + getBeliefbase().getBelief("utility").getFact() + " " + tid)));

		} // IF 2 STRING TOKENS

	}  // End of PlayerPlayPlan.body() method

	/** The program of the agent-plan for strategy choice
	 * 
	 * @param	G:		the game the player thinks he/she is playing
	 * @param	pid: 	the ID of the player/role we must assume in the game
	 * @param	tid: 	the ID of our type (e.g. pure hawk, tit4tat), i.e. the ID of our strategy selection mechanism/algorithm/program
	 * @param opponent: the local name of the opponent-player-agent (just to know with whom we are dealing with (again?))
	 * @return	the String corresponding to the int-value of the strategy
	 */
	private String chooseStrategy (Game G, int pid, int tid, String opponent) {

		// Set a default init-value...
		String returnval = "0";

		// Get the payoffs of player/role pid...
		double[][] myPayoffs = G.getPayoffs(pid);

		// Get the payoffs of the opponent...
		double[][] oppPayoffs = G.getPayoffs(1-pid);
		
		// Switch by player type...
		switch(tid) {

			// ------------------------------------------------------------------------------------
			case PlayerType.RANDOM:

				// Return a String of value "0" or "1" randomly (according to uniform distribution)
				returnval = Integer.toString(r.nextInt(2));

				break;

			// ------------------------------------------------------------------------------------
			case PlayerType.DOMINANT:

				if (pid == 0) {

					// If strategy 1 is dominant, then...
					if ((myPayoffs[1][0] >= myPayoffs[0][0]) && (myPayoffs[1][1] >= myPayoffs[0][1])) {

						returnval = "1";

					} else {

						returnval = "0";

					}

				} else { // PID == 1

					// If strategy 1 is dominant, then...
					if ((myPayoffs[0][1] >= myPayoffs[0][0]) && (myPayoffs[1][1] >= myPayoffs[1][0])) {

						returnval = "1";

					} else {

						returnval = "0";

					}			

				}

				break;

			// ------------------------------------------------------------------------------------
			case PlayerType.NASH:

				// returnval = "the strategy prescribed by a - possibly maximal - Nash-equilibrium of the game";

				break;

			// ------------------------------------------------------------------------------------
			case PlayerType.PARETO:

				// returnval = "the strategy prescribed by a - possibly maximal - Pareto-optimum of the game";

				break;

			// ------------------------------------------------------------------------------------		
			case PlayerType.TIT4TAT:

				// Let's be positive (by default)...
				returnval = "1";
				
				// Create an OQL (Object Query Language) expression to retrieve memories about opponents from the BeliefBase...
				IExpression query_memory = getExpression("query_memory");

				// Let's consider considering memories about this opponent...
				Tuple memory_about_opponent = (Tuple)query_memory.execute("$opponent", opponent);

				// If we've met this opponent before, then...
				if (memory_about_opponent != null) {

					// Get the second element of the Tuple: the action history (with me) of the player...
					Vector<String[]> opponents_action_history = (Vector<String[]>)memory_about_opponent.get(1);

					// --------------------------------------------------------------
					// The opponents_action_history is a Vector of observations about
					// a given opponent in chronological order. An observation is a
					// String[], which describes a memory of a play with the given
					// opponent. It consists of the following elements:
					// --------------------------------------------------------------
					// [0]	opponent_pid		The role of the opponent
					// [1]	opponent_sid		The strategy he/she played
					// [2]	opponent_payoff		The payoff he/she got
					// [3]	my_pid				My role
					// [4]	my_sid				My strategy
					// [5]	my_payoff			My payoff
					// --------------------------------------------------------------

					boolean is_symmetric = G.getSymmetry();
					
					// If the game is symmetric, then...
					if (is_symmetric) {
						
						// Return his previous action - 2nd element of the last observation...
						returnval = opponents_action_history.lastElement()[1];
						
					} else {
						
						String[] opp_obs;
						boolean found	= false;
						int opp_obs_num = opponents_action_history.size();
						
						// While not found the appropriate observation, and there are obs. left, then...
						while (!found && opp_obs_num > 0) {
							
							opp_obs_num--;
							opp_obs = opponents_action_history.elementAt(opp_obs_num);
							
							// If the opponent played the same role as now, then...
							if (opp_obs[0].equals(Integer.toString(1-pid))) {
							
								found		= true;
								returnval	= opp_obs[1];
								
							} // IF OPP. WAS IN THE SOME ROLE AS NOW
														
						} // WHILE NOT FOUND
						
					} // IF IS SYMMETRIC
					
				} // IF MEMORY ABOUT OPP. IS NOT NULL

				break;

			// ------------------------------------------------------------------------------------	
			case PlayerType.HAWK:
				
				returnval = "0";
				
				break;
			
			// ------------------------------------------------------------------------------------
			case PlayerType.DOVE:
				
				returnval = "1";
				
				break;

			// ------------------------------------------------------------------------------------				
			case PlayerType.TALK:
				
				returnval = "0";
				
				break;

			// ------------------------------------------------------------------------------------
			case PlayerType.SILENT:
				
				returnval = "1";
				
				break;
				
			// ------------------------------------------------------------------------------------
			case PlayerType.RECKLESS:
				
				returnval = "0";
				
				break;
			// ------------------------------------------------------------------------------------	
			case PlayerType.CHICKEN:
				
				returnval = "1";
				
				break;
			
			// ------------------------------------------------------------------------------------
			case PlayerType.OPERA:
				
				returnval = "0";
				
				break;
				
			// ------------------------------------------------------------------------------------	
			case PlayerType.FOOTBALL:
				
				returnval = "1";
				
				break;

			// ------------------------------------------------------------------------------------
			case PlayerType.DEFECT:
				
				returnval = "0";
				
				break;
				
			// ------------------------------------------------------------------------------------	
			case PlayerType.COOPERATE:
				
				returnval = "1";
				
				break;
				
			// ------------------------------------------------------------------------------------
			case PlayerType.GO:
				
				returnval = "0";
				
				break;
				
			// ------------------------------------------------------------------------------------
			case PlayerType.WAIT:
				
				returnval = "1";
				
				break;
				
			// ------------------------------------------------------------------------------------
			case PlayerType.HEADS:
				
				returnval = "0";
				
				break;
				
			// ------------------------------------------------------------------------------------
			case PlayerType.TAILS:
				
				returnval = "1";
				
				break;
				
		} // END OF SWITCH
		
		return returnval;

	} // End of PlayerPlayPlan.chooseStrategy() method

} // End of PlayerPlayPlan class
